﻿using System.Collections;
using System.Collections.Generic;
using _Combo;
using _Core;
using UnityEngine;
using UnityEngine.UI;

namespace Seayoo.ComboSDK.Windows.Account.View
{
    public class SeayooAccountPasswordLoginView : MonoBehaviour
    {
        public Button togglePasswordVisibilityBtn;
        public Button clearBtn;
        public Button setPasswordBtn;
        public InputField password;
        public Sprite invisibleImg;
        public Sprite visibleImg;
        public Text passwordText;
        private const int PASSWORD_MIN_LENGTH = 8;

        void Start()
        {
            password.contentType = InputField.ContentType.Password;
            togglePasswordVisibilityBtn.onClick.AddListener(TogglePasswordVisibility);
            password.onValueChanged.AddListener(OnInputFieldValueChanged);
            clearBtn.onClick.AddListener(ClearPassword);
            setPasswordBtn.onClick.AddListener(()=> {OpenSetPassword(Config.forgetPassword);});
            clearBtn.gameObject.SetActive(false);
            passwordText.text = I18n.T("login_reset_password_button");
        }

        void OnDestroy()
        {
            togglePasswordVisibilityBtn.onClick.RemoveListener(TogglePasswordVisibility);
            password.onValueChanged.RemoveListener(OnInputFieldValueChanged);
            clearBtn.onClick.RemoveListener(ClearPassword);
            setPasswordBtn.onClick.RemoveListener(()=> {OpenSetPassword(Config.forgetPassword);});
        }
        
        public bool CheckPassword()
        {
            if (string.IsNullOrEmpty(password.text))
            {
                UIController.Instance.ShowToast(I18n.T("password_required_toast"));
                return false;
            }
            if (password.text.Length < PASSWORD_MIN_LENGTH)
            {
                UIController.Instance.ShowToast(I18n.T("password_invalid_toast"));
                return false;
            }
            return true;
        }

        public void TogglePasswordVisibility()
        {
            if (password.inputType == InputField.InputType.Password)
            {
                password.inputType = InputField.InputType.Standard;
                togglePasswordVisibilityBtn.image.sprite = visibleImg;
            }
            else
            {
                password.inputType = InputField.InputType.Password;
                togglePasswordVisibilityBtn.image.sprite = invisibleImg;
            }

            password.ForceLabelUpdate();
        }

        private void OnInputFieldValueChanged(string value)
        {
            clearBtn.gameObject.SetActive(!string.IsNullOrEmpty(value));
        }

        private void ClearPassword()
        {
            password.text ="";;
        }

        private void OpenSetPassword(string url)
        {
#if UNITY_STANDALONE
            UIController.Instance.ShowWebView(url);
#endif
        }
    }
}