using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using _Core;
using _Combo;

namespace Seayoo.ComboSDK.Windows.Account.View
{
    public class SeayooAccountTokenLoginView : MonoBehaviour
    {

        public Button closeBtn;
        public Button enterGameBtn;
        public Button otherAccountBtn;
        public Dropdown dropdown;
        public Text loginText;
        public Text enterGameText;
        public Text otherAccountText;
        public GameObject logoImage;
        public Image mask;

        void Awake()
        {
            EventSystem.Register(this);
            loginText.text = I18n.T("login_title");
            enterGameText.text = I18n.T("login_submit_button");
            otherAccountText.text = I18n.T("login_other_accounts_button");
            gameObject.SetActive(false);
            if(GlobalParameters.Instance.SeayooAccountHideLogo)
            {
                logoImage.gameObject.SetActive(false); 
                loginText.gameObject.SetActive(true);
            }
        }

        void Start()
        {
            closeBtn.onClick.AddListener(OnClose);
            enterGameBtn.onClick.AddListener(OnEnterGame);
            otherAccountBtn.onClick.AddListener(OnLoginOtherAccount);
        }

        void OnDestroy()
        {
            closeBtn.onClick.RemoveListener(OnClose);
            enterGameBtn.onClick.RemoveListener(OnEnterGame);
            otherAccountBtn.onClick.RemoveListener(OnLoginOtherAccount);
            EventSystem.UnRegister(this);
        }

        [EventSystem.BindEvent]
        void OnGoToView(GoToViewCommand action) {
            if (action.viewType != SeayooAccountViewType.TOKEN) 
            {
                Hide();
                return;
            }
            
            Show();
        }

        public void ClickDropdowItem(string mobileMask)
        {
            Alert.Show(I18n.T("login_delete_credential_dialog_text"), I18n.T("login_delete_credential_dialog_title"), () =>
            {
                var option = dropdown.options.Find(o => string.Equals(o.text, mobileMask));
                dropdown.options.Remove(option);
                dropdown.Hide();
                dropdown.RefreshShownValue();
                var userId = UserHistoryManager.Instance.FindUserHistory(mobileMask).userId;
                DeleteUserHistoryCommand.Invoke(new DeleteUserHistoryCommand { userId = userId });
            });
        }

        public void OnEnterGame()
        {
            this.enterGameBtn.interactable = false;
            var user = UserHistoryManager.Instance.FindUserHistory(dropdown.value);
            TokenLoginCommand.Invoke(new TokenLoginCommand {
                userId = user.userId,
                token = user.token,
            });
        }

        public void OnClose()
        {
            CloseWindowCommand.Invoke(new CloseWindowCommand { userCancel = true });
        }

        public void Show()
        {
            this.gameObject.SetActive(true);

            mask.color = new Color(0f, 0f, 0f, 0.3f);
            Refresh();
        }

        [EventSystem.BindEvent]
        public void Refresh(RefreshViewCommand action = null)
        {

            var histories = UserHistoryManager.Instance.GetUsers();

            dropdown.ClearOptions();
            var options = new List<Dropdown.OptionData>(histories.Count);
            histories.ForEach(record =>
            {
                options.Add(new Dropdown.OptionData(record.mobileMask));
            });
            dropdown.options = options;
            this.enterGameBtn.interactable = true;
        }

        public void Hide()
        {
            this.gameObject.SetActive(false);
        }

        public void OnLoginOtherAccount()
        {
            GoToViewCommand.Invoke(new GoToViewCommand {
                viewType = SeayooAccountViewType.PHONE,
            });
        }
    }
}
