using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using _Combo;
using _Combo.Events;
using _Core;
using Seayoo.ComboSDK.Windows.Store.Networking;
using Seayoo.ComboSDK.Windows.Store.Tools;
using UnityEngine;
using UnityEngine.UI;

namespace Seayoo.ComboSDK.Windows.Store.View
{
    public class QRCodePayView : MonoBehaviour
    {
        public Button cancelBtn;
        public Button wxBtn;
        public Button alipayBtn;
        public Button sandboxBtn;
        public GameObject expiredPanel;
        public GameObject sandboxPanel;
        public GameObject qrAndTipPanel;
        public GameObject groupBtnPanel;
        public RectTransform sandboxRectTransform;
        public Text titleText;
        public Text wxBtnText;
        public Text alipayBtnText;
        public Text sandboxBtnText;
        public Text productNameText;
        public Text productValueText;
        public Text payTipText;
        public HyperlinkText reorderText;
        public Text expiredText;
        public Image expiredImage;
        public RawImage QRCodeImg;
        public Sprite selectedSprite;
        public Sprite normalSprite;
        private string[] methods;
        private bool stopWeixinPolling = false;
        private bool stopAlipayPolling = false;
        private bool isWeixinPolling = false;
        private bool isAlipayPolling = false;
        private long pollingTime;
        private long weixinTargetTime;
        private long alipayTargetTime;
        private string weixinUrl;
        private string alipayUrl;
        private Order order;
        private string orderToken;

        private QRCodeTool codeTool = new QRCodeTool();
        private string method = SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.WEIXIN_NATIVE);
        void Awake()
        {
            EventSystem.Register(this);
        }

        void Start()
        {
            titleText.text = I18n.T("payment_qrcode_title");
            wxBtnText.text = I18n.T("weixin_pay_button");
            alipayBtnText.text = I18n.T("alipay_button");
            sandboxBtnText.text = I18n.T("sandbox_payment_button");
            expiredText.text = I18n.T("payment_qrcode_expired_text_above_qrcode");
            reorderText.text = I18n.T("payment_qrcode_expired_text");
            cancelBtn.onClick.AddListener(OnClose);
            wxBtn.onClick.AddListener(OnWeixinPay);
            alipayBtn.onClick.AddListener(OnAlipayPay);
            sandboxBtn.onClick.AddListener(OnSandboxPay);
            Init();
        }

        void OnDestroy()
        {
            EventSystem.UnRegister(this);
            cancelBtn.onClick.RemoveListener(OnClose);
            wxBtn.onClick.RemoveListener(OnWeixinPay);
            alipayBtn.onClick.RemoveListener(OnAlipayPay);
            sandboxBtn.onClick.RemoveListener(OnSandboxPay);
        }

        public void OnClose()
        {
            stopWeixinPolling = true;
            stopAlipayPolling = true;
            SeayooStoreClient.QueryPayment(orderToken, method, result =>
            {
                if(result.Data.paymentStatus == "pending")
                {
                    PurchaseCancelEvent.Invoke();
                }
                else if(result.Data.paymentStatus == "paid")
                {
                    PurchaseSuccessEvent.Invoke(new PurchaseSuccessEvent
                    {
                        orderId = order.orderId
                    });
                }
                else
                {
                    var ie = new InternalError(I18n.T("iap_payment_failed"), new StoreError(order, "seayoo_store"));
                    PurchaseFailedEvent.Invoke(new PurchaseFailedEvent{ error = ie });
                }
                ClosePayViewCommand.Invoke(new ClosePayViewCommand());
            }, (err) => {
                PurchaseFailedEvent.Invoke(new PurchaseFailedEvent{ error = err });
                ClosePayViewCommand.Invoke(new ClosePayViewCommand());
            });
        }

        public void OnNewOrder()
        {
            var error = new InternalError(I18n.T("order_expired_toast"), new StoreError(order, "seayoo_store", enableTracking: false));
            PurchaseFailedEvent.Invoke(new PurchaseFailedEvent{ error = error });
            ClosePayViewCommand.Invoke(new ClosePayViewCommand());
            stopWeixinPolling = true;
            stopAlipayPolling = true;
        }

        public void OnWeixinPay()
        {
            qrAndTipPanel.gameObject.SetActive(true);
            method = SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.WEIXIN_NATIVE);
            wxBtn.GetComponent<Image>().sprite = selectedSprite;
            alipayBtn.GetComponent<Image>().sprite = normalSprite;
            sandboxBtn.GetComponent<Image>().sprite = normalSprite;
            payTipText.text = I18n.T("weixin_pay_qrcode_text");
            if(string.IsNullOrEmpty(weixinUrl))
            {
                SeayooStoreManager.Instance.RequestPayment(orderToken, method);
                return;
            }
            ShowQRCode(method, weixinUrl);
        }

        public void OnAlipayPay()
        {
            qrAndTipPanel.gameObject.SetActive(true);
            method = SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.ALIPAY_QR_CODE);
            alipayBtn.GetComponent<Image>().sprite = selectedSprite;
            wxBtn.GetComponent<Image>().sprite = normalSprite;
            sandboxBtn.GetComponent<Image>().sprite = normalSprite;
            payTipText.text = I18n.T("alipay_qrcode_text");
            if(string.IsNullOrEmpty(alipayUrl))
            {
                SeayooStoreManager.Instance.RequestPayment(orderToken, method);
                return;
            }
            ShowQRCode(method, alipayUrl);
        }

        public void OnSandboxPay()
        {
            method = SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.SEAYOO_SANDBOX);
            sandboxBtn.GetComponent<Image>().sprite = selectedSprite;
            wxBtn.GetComponent<Image>().sprite = normalSprite;
            alipayBtn.GetComponent<Image>().sprite = normalSprite;
            GoToViewCommand.Invoke(new GoToViewCommand(){
                type = PayViewType.PAY_SANDBOX,
                productName = productNameText.text,
                productValue = productValueText.text
            });
        }

        [EventSystem.BindEvent]
        public void OnQueryResult(QueryPaymentResultSCommand action)
        {
            if (action.status == PaymentStatus.PAID)
            {
                stopWeixinPolling = true;
                stopAlipayPolling = true;
                GoToViewCommand.Invoke(new GoToViewCommand{
                    type = PayViewType.PAY_SUCCESS,
                    order = order,
                    productName = productNameText.text,
                    productValue = productValueText.text
                });
            }
        }

        [EventSystem.BindEvent]
        public void ShowTextInfo(ProductInfoCommand action)
        {
            orderToken = action.orderToken;
            methods = action.seayooPurchaseOptions.paymentMethods;
            order = action.order;
            productNameText.text = action.order.productName;
            productValueText.text = "￥" + (action.order.amount / 100.0).ToString("F2");
        }

        [EventSystem.BindEvent]
        public void ShowPayView(GoToViewCommand action)
        {
            if(action.type != PayViewType.PAY)
            {
                gameObject.SetActive(false);
                return;
            }
            gameObject.SetActive(true);
            productNameText.text = action.productName;
            productValueText.text = action.productValue;
        }

        [EventSystem.BindEvent]
        public void ShowQRCode(QRCodeCommand action)
        {
            if (codeTool.isEncodeing() == false) {
                if(action.paymentMethod == PaymentMethods.WEIXIN_NATIVE)
                {
                    weixinUrl = action.url;
                    weixinTargetTime = action.timeExpire;
                    pollingTime = weixinTargetTime;
                    isWeixinPolling = true;
                    isAlipayPolling = false;
                    StartCoroutine("WeixinPolling");
                }
                else if(action.paymentMethod == PaymentMethods.ALIPAY_QR_CODE)
                {
                    alipayUrl = action.url;
                    alipayTargetTime = action.timeExpire;
                    pollingTime = alipayTargetTime;
                    isAlipayPolling = true;
                    isWeixinPolling = false;
                    StartCoroutine("AlipayPolling");
                    StopCoroutine("WeixinPolling");
                }
                codeTool.EncodeQRCode(action.url, new Vector2Int(300, 300));
                StartCoroutine(getQRCode());
            }
        }

        private void Init()
        {
            if (methods == null || methods.Length == 0)
            {
                PurchaseFailedEvent.Invoke(new PurchaseFailedEvent { error = new InternalError(I18n.T("iap_payment_method_unavailable_error"), new StoreError(order, "seayoo_store"))});
                ClosePayViewCommand.Invoke(new ClosePayViewCommand());
                return;
            }

            bool hasWeixinNative = methods.Contains("weixin_native");
            bool hasAlipayQrCode = methods.Contains("alipay_qr_code");
            bool hasSeayooSandbox = methods.Contains("seayoo_sandbox");

            wxBtn.gameObject.SetActive(hasWeixinNative);
            alipayBtn.gameObject.SetActive(hasAlipayQrCode);
            sandboxBtn.gameObject.SetActive(hasSeayooSandbox);

            if(hasSeayooSandbox)
            {
                qrAndTipPanel.gameObject.SetActive(false);
                sandboxPanel.gameObject.SetActive(true);
                if(methods.Length == 1)
                {
                    groupBtnPanel.SetActive(false);
                }
                else if(methods.Length == 2)
                {
                    sandboxRectTransform.sizeDelta = new Vector2(280f, sandboxRectTransform.sizeDelta.y);
                }
            }
            else
            {
                sandboxPanel.gameObject.SetActive(false);
                qrAndTipPanel.gameObject.SetActive(true);
                if (!hasWeixinNative)
                {
                    method = SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.ALIPAY_QR_CODE);
                    OnAlipayPay();
                }
                else if (!hasAlipayQrCode)
                {
                    method = SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.WEIXIN_NATIVE);
                    OnWeixinPay();
                }
                else
                {
                    OnWeixinPay();
                }
            }
            
        }

        private void ShowQRCode(string method, string url)
        {
            if (codeTool.isEncodeing() == false) {
                codeTool.EncodeQRCode(url, new Vector2Int(300, 300));
                StartCoroutine(getQRCode());
                if(method == SeayooStoreManager.Instance.GetPaymentMethod(PaymentMethods.WEIXIN_NATIVE))
                {
                    if(isWeixinPolling)
                    {
                        return;
                    }
                    StartCoroutine("WeixinPolling");
                    StopCoroutine("AlipayPolling");
                    isWeixinPolling = true;
                    isAlipayPolling = false;
                }
                else
                {
                    if(isAlipayPolling)
                    {
                        return;
                    }
                    StartCoroutine("AlipayPolling");
                    StopCoroutine("WeixinPolling");
                    isAlipayPolling = true;
                    isWeixinPolling = false;
                }
            }
        }


        IEnumerator getQRCode() {
            while (true) {
                if (codeTool.GetQRimage() != null) {
                    QRCodeImg.texture = codeTool.GetQRimage();
                    break;
                }
                yield return 0;
            }
        }

        private IEnumerator WeixinPolling()
        {
            while (!stopWeixinPolling)
            {
                long currentTimestamp = DateTimeOffset.Now.ToUnixTimeSeconds();

                // 判断是否到达指定时间戳的1分钟后，如果到达则停止轮询
                if (currentTimestamp >= pollingTime + 60)
                {
                    stopWeixinPolling = true;
                }
                if (currentTimestamp >= weixinTargetTime)
                {
                    // 二维码过期
                    expiredImage.gameObject.SetActive(true);
                    expiredText.gameObject.SetActive(true);
                    payTipText.gameObject.SetActive(false);
                    expiredPanel.SetActive(true);
                    StartCoroutine(SetHyperlinkActionsDelayed()); 
                }
                SeayooStoreManager.Instance.QueryPayment(orderToken, method);
                yield return new WaitForSeconds(2f);
            }
        }

        private IEnumerator AlipayPolling()
        {
            while (!stopAlipayPolling)
            {
                long currentTimestamp = DateTimeOffset.Now.ToUnixTimeSeconds();

                // 判断是否到达指定时间戳的1分钟后，如果到达则停止轮询
                if (currentTimestamp >= pollingTime + 60)
                {
                    stopAlipayPolling = true;
                }
                if (currentTimestamp >= alipayTargetTime)
                {
                    // 二维码过期
                    expiredImage.gameObject.SetActive(true);
                    expiredText.gameObject.SetActive(true);
                    payTipText.gameObject.SetActive(false);
                    expiredPanel.SetActive(true);
                    StartCoroutine(SetHyperlinkActionsDelayed()); 
                }
                else
                {
                    expiredImage.gameObject.SetActive(false);
                    expiredText.gameObject.SetActive(false);
                    payTipText.gameObject.SetActive(true);
                    expiredPanel.SetActive(false);
                }
                SeayooStoreManager.Instance.QueryPayment(orderToken, method);
                yield return new WaitForSeconds(2f);
            }
        }

        private IEnumerator SetHyperlinkActionsDelayed()
        {
            // 等待一帧，确保 OnPopulateMesh 执行
            yield return null;

            reorderText.SetHyperlinkClickActions(new Dictionary<string, System.Action<string>>{
                {"reorder", (link)=>{OnNewOrder();}}
            });
        }

    }
}