﻿using System;
using System.Reflection;

namespace _Core
{
    internal static class TapTapAntiAddiction
    {
        public static Type tapAntiAddictionType = Assembly.Load("TapTap.AntiAddiction").GetType("TapTap.AntiAddiction.AntiAddictionUIKit");
        public static Type tapAntiAddictionConfigType = Assembly.Load("TapTap.AntiAddiction").GetType("TapTap.AntiAddiction.Model.AntiAddictionConfig");
        public static void Init(string clientID, bool showSwitchAccount, bool useAgeRange)
        {
            if(tapAntiAddictionType == null)
            {
                Log.W("Failed to get type AntiAddictionUIKit");
            }
            else
            {
                MethodInfo methodWithThreeParams = tapAntiAddictionType.GetMethod("Init", 
                    BindingFlags.Static | BindingFlags.Public, 
                    null, 
                    new Type[] { tapAntiAddictionConfigType }, 
                    null);
                if (methodWithThreeParams != null)
                {
                    // 创建并初始化 AntiAddictionConfig 实例
                    object configInstance = Activator.CreateInstance(tapAntiAddictionConfigType);
                    if (configInstance != null)
                    {
                        // 获取并设置 AntiAddictionConfig 的属性
                        FieldInfo gameIdProperty = tapAntiAddictionConfigType.GetField("gameId");
                        FieldInfo showSwitchAccountProperty = tapAntiAddictionConfigType.GetField("showSwitchAccount");
                        FieldInfo useAgeRangeProperty = tapAntiAddictionConfigType.GetField("useAgeRange");

                        if (gameIdProperty != null && showSwitchAccountProperty != null && useAgeRangeProperty != null)
                        {
                            gameIdProperty.SetValue(configInstance, clientID);
                            showSwitchAccountProperty.SetValue(configInstance, showSwitchAccount);
                            useAgeRangeProperty.SetValue(configInstance, useAgeRange);

                            // 调用 Init 方法
                            methodWithThreeParams.Invoke(null, new object[] { configInstance });
                            Console.WriteLine("Method Init invoked successfully.");
                        }
                        else
                        {
                            Console.WriteLine("Failed to get properties of AntiAddictionConfig");
                        }
                    }
                }
            }
        }

        public static void StartupWithTapTap(string userIdentifier)
        {
            if(tapAntiAddictionType == null)
            {
                Log.W("Failed to get type AntiAddictionUIKit");
            }
            else
            {
                MethodInfo methodWithThreeParams = tapAntiAddictionType.GetMethod("StartupWithTapTap", 
                    BindingFlags.Static | BindingFlags.Public, 
                    null, 
                    new Type[] { typeof(string) }, 
                    null);
                if (methodWithThreeParams != null)
                {
                    methodWithThreeParams.Invoke(null, new object[] {userIdentifier});
                }
                else
                {
                    Log.W("Failed to get method Init");
                }
            }
        }

        public static void SetAntiAddictionCallback(Action<int, string> callback)
        {
            if(tapAntiAddictionType == null)
            {
                Log.W("Failed to get type AntiAddictionUIKit");
            }
            else
            {
                MethodInfo setCallbackMethod = tapAntiAddictionType.GetMethod("SetAntiAddictionCallback", 
                    BindingFlags.Public | BindingFlags.Static);
                if (setCallbackMethod != null)
                {
                    setCallbackMethod.Invoke(null, new object[] { callback });
                    Console.WriteLine("Callback 设置成功");
                }
                else
                {
                    Console.WriteLine("SetAntiAddictionCallback 方法未找到");
                }
            }
        }

        public static int GetAgeRange()
        {
            if(tapAntiAddictionType == null)
            {
                Log.W("Failed to get type AntiAddictionUIKit");
                return -1;
            }
            else
            {
                PropertyInfo ageRangeField = tapAntiAddictionType.GetProperty("AgeRange", BindingFlags.Public | BindingFlags.Static);
                if (ageRangeField != null)
                {
                    // 获取 AgeRange 字段的值
                    int ageRangeValue = (int)ageRangeField.GetValue(null);
                    return ageRangeValue;
                }
                else
                {
                    Console.WriteLine("AgeRange 字段未找到");
                    return -1;
                }
            }
        }

        public static void Exit()
        {
            if(tapAntiAddictionType == null)
            {
                Log.W("Failed to get type AntiAddictionUIKit");
            }
            else
            {
                MethodInfo methodWithThreeParams = tapAntiAddictionType.GetMethod("Exit", BindingFlags.Static | BindingFlags.Public);
                if (methodWithThreeParams != null)
                {
                    methodWithThreeParams.Invoke(null, null);
                }
                else
                {
                    Log.W("Failed to get method Init");
                }
            }
        }
    }
}