using System;
using System.Reflection;
using System.Threading.Tasks;
using _Combo;
using Newtonsoft.Json;
using UnityEngine;

namespace _Core
{
    internal static class TapTapLogin
    {
        public static Type tapLoginType = Assembly.Load("TapTap.Login.Runtime").GetType("TapTap.Login.TapLogin");
        public static Type tapExceptionType = Assembly.Load("TapTap.Common.Runtime").GetType("TapTap.Common.TapException");
        public static Type tapErrorCodeType = Assembly.Load("TapTap.Common.Runtime").GetType("TapTap.Common.TapErrorCode");
        public static void Init(string clientID, bool isCn, bool roundCorner)
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
            }
            else
            {
                MethodInfo methodWithThreeParams = tapLoginType.GetMethod("Init", 
                    BindingFlags.Static | BindingFlags.Public, 
                    null, 
                    new Type[] { typeof(string), typeof(bool), typeof(bool) }, 
                    null);
                if (methodWithThreeParams != null)
                {
                    methodWithThreeParams.Invoke(null, new object[] { clientID, isCn, roundCorner });
                }
                else
                {
                    Log.W("Failed to get method Init");
                }
            }
        }

        public static async Task LoginAsync(Action<TapTapLoginResult> onSuccess, Action<InternalError> onError)
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
            }
            else
            {
                MethodInfo methodNoParam = tapLoginType.GetMethod
                (
                    "Login", 
                    BindingFlags.Static | BindingFlags.Public, 
                    null, 
                    Type.EmptyTypes, 
                    null
                );

                if (methodNoParam != null)
                {
                    try
                    {
                        Task result = (Task)methodNoParam.Invoke(null, null);
                        await result;

                        PropertyInfo resultProperty = result.GetType().GetProperty("Result");
                        object accessToken = resultProperty.GetValue(result);
                        MethodInfo toJsonMethod = accessToken.GetType().GetMethod("ToJson");
                        string json = (string)toJsonMethod.Invoke(accessToken, null);
                        
                        Log.I($"TapTap login success, accessToken: {json}");
                        onSuccess?.Invoke(JsonConvert.DeserializeObject<TapTapLoginResult>(json));
                    }
                    catch (Exception e)
                    {
                        if (e.GetType() == tapExceptionType)
                        {
                            FieldInfo codeField = tapExceptionType.GetField("code", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);
                            int codeValue = (int)codeField.GetValue(e);
                            FieldInfo messageField = tapExceptionType.GetField("message", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);
                            string messageValue = (string)messageField.GetValue(e);

                            Log.I($"encounter exception:{codeValue} message:{messageValue}");

                            var errorCodeCancel = Enum.Parse(tapErrorCodeType, "ERROR_CODE_LOGIN_CANCEL");

                            if (codeValue.Equals(Convert.ToInt32(errorCodeCancel)))
                            {
                                onError(new InternalError(new UserCancelled()));
                                return;
                            }
                            
                            var loginError = new LoginError("taptap", thirdPartyCode: codeValue, thirdPartyMessage: messageValue);
                            onError(new InternalError(messageValue, loginError));
                        }
                        else
                        {
                            Log.E(e);
                            onError(new InternalError(e.Message, new LoginError("taptap")));
                        }
                    }
                }
                else
                {
                    Log.W("Failed to get method Init");
                    onError(new InternalError("Failed to get method Init", new LoginError("taptap")));
                }
            }
        }

        public static void Logout()
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
            }
            else
            {
                MethodInfo logoutMethod = tapLoginType.GetMethod("Logout", BindingFlags.Static | BindingFlags.Public);

                if (logoutMethod != null)
                {
                    logoutMethod.Invoke(null, null);
                    Log.I("Logout method invoked successfully.");
                }
                else
                {
                    Log.E("Logout method not found.");
                }
            }
        }

        public async static Task GetAccessToken(Action<TapTapLoginResult> onSuccess, Action<InternalError> onError)
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
            }
            else
            {
                try
                {
                    MethodInfo getAccessTokenMethod = tapLoginType.GetMethod("GetAccessToken", BindingFlags.Static | BindingFlags.Public);

                    if (getAccessTokenMethod != null)
                    {
                        Task task = (Task)getAccessTokenMethod.Invoke(null, null);
                        await task;

                        PropertyInfo resultProperty = task.GetType().GetProperty("Result");
                        object accessToken = resultProperty.GetValue(task);

                        MethodInfo toJsonMethod = accessToken.GetType().GetMethod("ToJson");
                        string json = (string)toJsonMethod.Invoke(accessToken, null);

                        Log.I($"TapTap login success, accessToken: {json}");
                        onSuccess?.Invoke(JsonConvert.DeserializeObject<TapTapLoginResult>(json));
                    }
                    else
                    {
                        Log.E("GetAccessToken method not found.");
                        onError(new InternalError("GetAccessToken method not found.", new LoginError("taptap")));
                    }
                }
                catch (Exception e)
                {
                    onError(new InternalError(e.Message, new LoginError("taptap")));
                }
            }
        }

        public async static Task<TapTapProfile> GetProfile()
        {
            if(tapLoginType == null)
            {
                Log.W("Failed to get type TapLogin");
                return null;
            }
            else
            {
                MethodInfo profileMethod = tapLoginType.GetMethod("GetProfile", BindingFlags.Static | BindingFlags.Public);
                if (profileMethod != null)
                {
                    Task profileTask = (Task)profileMethod.Invoke(null, null);
                    await profileTask;

                    PropertyInfo resultProperty = profileTask.GetType().GetProperty("Result");
                    object accessToken = resultProperty.GetValue(profileTask);
                    MethodInfo toJsonMethod = accessToken.GetType().GetMethod("ToJson");
                    string json = (string)toJsonMethod.Invoke(accessToken, null);

                    
                    return JsonConvert.DeserializeObject<TapTapProfile>(json);
                }
                else
                {
                    Console.WriteLine("FetchProfile 方法未找到");
                    return null;
                }
            }
        }
    }
}