using System;
using System.Collections.Generic;
using Newtonsoft.Json;

namespace Combo.Platform
{
    internal static class Native
    {
        internal static void Setup(SetupOptions opts, Action<Result<SetupResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.Setup(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.Setup(opts, callback);
#elif UNITY_STANDALONE || UNITY_EDITOR
            NativeWindows.Setup(opts, callback);
#endif
        }

        internal static void Login(Action<Result<LoginResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.Login(callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.Login(callback);
#elif UNITY_STANDALONE || UNITY_EDITOR
            NativeWindows.Login(callback);
#endif
        }

        internal static LoginInfo GetLoginInfo()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return NativeAndroid.GetLoginInfo();
#elif UNITY_IOS && !UNITY_EDITOR
            return NativeIOS.GetLoginInfo();
#elif UNITY_STANDALONE || UNITY_EDITOR
            return NativeWindows.GetLoginInfo();
#else
            return null;
#endif
        }

        internal static void Logout(Action<Result<LogoutResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.Logout(callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.Logout(callback);
#elif UNITY_STANDALONE || UNITY_EDITOR
            NativeWindows.Logout(callback);
#endif
        }

        internal static void Purchase(PurchaseOptions opts, Action<Result<PurchaseResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.Purchase(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.Purchase(opts, callback);
#elif UNITY_STANDALONE || UNITY_EDITOR
            NativeWindows.Purchase(opts, callback);
#endif
        }

        internal static void PreloadAd(PreloadAdOptions opts, Action<Result<PreloadAdResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.PreloadAd(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.PreloadAd(opts, callback);
#elif UNITY_STANDALONE || UNITY_EDITOR
#endif
        }

        internal static void ShowAd(ShowAdOptions opts, Action<Result<ShowAdResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.ShowAd(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.ShowAd(opts, callback);
#elif UNITY_STANDALONE || UNITY_EDITOR
#endif
        }

        internal static void Share(ShareOptions opts, Action<Result<ShareResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.Share(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.Share(opts, callback);
#elif UNITY_STANDALONE || UNITY_EDITOR
#endif
        }

        internal static void ReportCreateRole(RoleInfo roleInfo) {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.ReportCreateRole(roleInfo);
#elif UNITY_IOS && !UNITY_EDITOR
#elif UNITY_STANDALONE || UNITY_EDITOR
#endif
        }

        internal static void ReportEnterGame(RoleInfo roleInfo) {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.ReportEnterGame(roleInfo);
#elif UNITY_IOS && !UNITY_EDITOR
#elif UNITY_STANDALONE || UNITY_EDITOR
#endif
        }

#region
        internal static void OnKickOut(Action<Result<KickOutResult>> callback) {
#if UNITY_ANDROID
            NativeAndroid.OnKickOut(callback);
#elif UNITY_IOS
            NativeIOS.OnKickOut(callback);
#elif UNITY_STANDALONE
            NativeWindows.OnKickOut(callback);
#endif  
        }

        internal static bool IsFeatureAvailable(Feature feature) {
#if UNITY_ANDROID
            return NativeAndroid.IsFeatureAvailable(feature);
#elif UNITY_IOS
            return NativeIOS.IsFeatureAvailable(feature);
#elif UNITY_STANDALONE || UNITY_EDITOR
            return NativeWindows.IsFeatureAvailable(feature);
#endif  
        }

        internal static void UpdateGame(UpdateGameOptions opts, Action<Result<UpdateGameResult>> callback) {
#if UNITY_ANDROID
            NativeAndroid.UpdateGame(new UpdateGameOptions(), callback);
#endif  
        }

        internal static ShareTarget[] GetAvailableShareTargets() {
#if UNITY_ANDROID && !UNITY_EDITOR
            return NativeAndroid.GetAvailableShareTargets();
#elif UNITY_IOS && !UNITY_EDITOR
            return NativeIOS.GetAvailableShareTargets();
#elif UNITY_STANDALONE || UNITY_EDITOR
            return new ShareTarget[]{ };
#endif
        }

        internal static string GetVariant() {
#if UNITY_ANDROID
            return NativeAndroid.GetVariant();
#endif 
            return null;
        }

        internal static string GetSubvariant() {
#if UNITY_ANDROID
            return NativeAndroid.GetSubvariant();
#endif 
            return null;
        }

        internal static void GetDownloadUrl(Action<Result<DownloadUrlResult>> callback) {
#if UNITY_ANDROID
            NativeAndroid.GetDownloadUrl(callback);
#elif UNITY_IOS
            NativeIOS.GetDownloadUrl(callback);
#elif UNITY_STANDALONE
            NativeWindows.GetDownloadUrl(callback);
#endif 
        }

        internal static void Quit(Action<Result<QuitResult>> callback) {
#if UNITY_ANDROID
            NativeAndroid.Quit(callback);
#endif 
        }
#endregion

        internal static void KeepRenderingOnPause() {
#if UNITY_ANDROID
            NativeAndroid.KeepRenderingOnPause();
#endif 
        }

        internal static string GetGameId() {
            return Configuration.Instance.GameId;
        }

        internal static string GetDistro()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return NativeAndroid.GetDistro();
#elif UNITY_IOS && !UNITY_EDITOR
            return NativeIOS.GetDistro();
#elif UNITY_STANDALONE || UNITY_EDITOR
            return NativeWindows.GetDistro();
#endif
        }

        internal static string GetVersion()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return NativeAndroid.GetVersion();
#elif UNITY_IOS && !UNITY_EDITOR
            return NativeIOS.GetVersion();
#elif UNITY_STANDALONE || UNITY_EDITOR
            return NativeWindows.GetVersion();
#endif
        }

        internal static T GetParameters<T>()
        {
            Dictionary<string, string> parameters;
#if UNITY_ANDROID && !UNITY_EDITOR
            parameters = NativeAndroid.GetParameters();
#elif UNITY_IOS && !UNITY_EDITOR
            parameters = NativeIOS.GetParameters();
#elif UNITY_STANDALONE || UNITY_EDITOR
            parameters = NativeWindows.GetParameters();
#endif
            parameters = parameters ?? new Dictionary<string, string>();
            return JsonConvert.DeserializeObject<T>(JsonConvert.SerializeObject(parameters));
        }

        internal static string GetDeviceId() {
#if UNITY_ANDROID && !UNITY_EDITOR
            return NativeAndroid.GetDeviceId();
#elif UNITY_IOS && !UNITY_EDITOR
            return NativeIOS.GetDeviceId();
#elif UNITY_STANDALONE || UNITY_EDITOR
            return NativeWindows.GetDeviceId();
#endif
        }

        internal static void OpenAppSettings() {
#if UNITY_ANDROID
            NativeAndroid.OpenAppSettings();
#elif UNITY_IOS
            NativeIOS.OpenAppSettings();
#endif 
        }

        internal static string UserId
        {
            get
            {
#if UNITY_ANDROID
                return NativeAndroid.GetUserId();
#elif UNITY_IOS
                return NativeIOS.GetUserId();
#elif UNITY_STANDALONE
                return NativeWindows.GetUserId();
#endif

            }
        }

        internal static void ManageAccount() 
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.ManageAccount();
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.ManageAccount();
#elif UNITY_STANDALONE
            NativeWindows.ManageAccount();
#endif           
        }

        internal static void ChangePassword() 
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.ChangePassword();
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.ChangePassword();
#elif UNITY_STANDALONE
            NativeWindows.ChangePassword();
#endif                             
        }

        internal static void DeleteAccount() 
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.DeleteAccount();
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.DeleteAccount();
#elif UNITY_STANDALONE
            NativeWindows.DeleteAccount();
#endif                             
        }

        internal static void ContactSupport() 
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.ContactSupport();
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.ContactSupport();
#elif UNITY_STANDALONE
            NativeWindows.ContactSupport();
#endif                             
        }

        internal static void OpenGameUrl(GameUrl url)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.OpenGameUrl(url);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.OpenGameUrl(url);
#elif UNITY_STANDALONE
            NativeWindows.OpenGameUrl(url);
#endif                  
        }

        internal static void OpenShortLink(string shortLink, Dictionary<string, string> gameData, Action<Result<OpenShortLinkResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.OpenShortLink(shortLink, gameData, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.OpenShortLink(shortLink, gameData, callback);
#elif UNITY_STANDALONE
            NativeWindows.OpenShortLink(shortLink, gameData, callback);
#endif   
        }

        internal static void CheckAnnouncements(CheckAnnouncementsOptions opts, Action<Result<CheckAnnouncementsResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.CheckAnnouncements(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.CheckAnnouncements(opts, callback);
#elif UNITY_STANDALONE
            NativeWindows.CheckAnnouncements(opts, callback);
#endif
        }

        internal static void OpenAnnouncements(OpenAnnouncementsOptions opts, Action<Result<OpenAnnouncementsResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.OpenAnnouncements(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.OpenAnnouncements(opts, callback);
#elif UNITY_STANDALONE
            NativeWindows.OpenAnnouncements(opts, callback);
#endif
        }

        internal static void Complain(ComplainOptions opts, Action<Result<ComplainResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.Complain(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.Complain(opts, callback);
#elif UNITY_STANDALONE
            NativeWindows.Complain(opts, callback);
#endif
        }

        internal static void PromoPseudoPurchase(PromoPseudoPurchaseOptions opts)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.PromoPseudoPurchase(opts);
#endif
        }

        internal static LanguagePreference LanguagePreference
        {
            get
            {
#if UNITY_ANDROID
                return NativeAndroid.GetLanguagePreference();
#elif UNITY_IOS
                return NativeIOS.GetLanguagePreference();
#elif UNITY_STANDALONE
                return NativeWindows.GetLanguagePreference();
#endif
            }

            set
            {
#if UNITY_ANDROID
                NativeAndroid.SetLanguagePreference(value);
#elif UNITY_IOS
                NativeIOS.SetLanguagePreference(value);
#elif UNITY_STANDALONE
                NativeWindows.SetLanguagePreference(value);
#endif
            }
        }

        internal static string LanguageCode
        {
            get
            {
#if UNITY_ANDROID
                return NativeAndroid.GetLanguageCode();
#elif UNITY_IOS
                return NativeIOS.GetLanguageCode();
#elif UNITY_STANDALONE
                return NativeWindows.GetLanguageCode();
#endif
            } 
        }

        internal static bool ResetGuest()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return NativeAndroid.ResetGuest();
#elif UNITY_IOS && !UNITY_EDITOR
            return NativeIOS.ResetGuest();
#elif UNITY_STANDALONE || UNITY_EDITOR
            return false;
#endif
        }

        internal static void RedeemGiftCode(RedeemGiftCodeOptions opts, Action<Result<RedeemGiftCodeResult>> callback)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            NativeAndroid.RedeemGiftCode(opts, callback);
#elif UNITY_IOS && !UNITY_EDITOR
            NativeIOS.RedeemGiftCode(opts, callback);
#elif UNITY_STANDALONE
            NativeWindows.RedeemGiftCode(opts, callback);
#endif
        }
    }
}
